"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toRendableTransfer = void 0;

var _utils = require("./utils");

const dateFormat = "D MMMM YYYY • HH:mm";

const toRendablePrivateTransfer = (intl, inscription) => {
  const {
    id,
    price,
    product: {
      duration,
      comment,
      paxplan,
      pickup,
      pickupRT,
      departure,
      destination
    }
  } = inscription;
  const paxCount = paxplan.reduce((acc, pax) => acc + pax.value, 0);
  const pickupDate = new Date(pickup.date);
  const pickupEndDate = new Date(pickupDate).setMinutes(pickupDate.getMinutes() + duration);
  const pickupRTDate = pickupRT ? new Date(pickupRT.date) : undefined;
  const pickupRTEndDate = pickupRTDate ? new Date(pickupRTDate).setMinutes(pickupRTDate.getMinutes() + duration) : undefined;
  return {
    id,
    label: intl.formatMessage({
      id: "transfer.private"
    }),
    pax: `${paxCount} ${intl.formatMessage({
      id: "transfer.people"
    })}`,
    price: (0, _utils.formatPrice)(intl, price),
    subLabel: [{
      type: "normal",
      text: pickupRT ? intl.formatMessage({
        id: "transfer.roundTrip"
      }) : intl.formatMessage({
        id: "transfer.oneWay"
      })
    }],
    details: [comment ? {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.comment"
      }),
      value: comment
    } : undefined, pickupRT ? {
      type: "title",
      label: intl.formatMessage({
        id: "transfer.forwardPath"
      })
    } : undefined, {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.departure"
      }),
      value: `${departure.label} • ${(0, _utils.format)(pickupDate, intl.locale, dateFormat)}`
    }, {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.arrival"
      }),
      value: `${destination.label}, ${pickup.destinationAddress} • ${(0, _utils.format)(pickupEndDate, intl.locale, dateFormat)}`
    }, {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.flightNumber"
      }),
      value: pickup.flightNumber
    }, pickupRT ? {
      type: "title",
      label: intl.formatMessage({
        id: "transfer.returnTrip"
      })
    } : undefined, pickupRT ? {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.departure"
      }),
      value: `${destination.label}, ${pickupRT.departureAddress} • ${(0, _utils.format)(pickupRTDate, intl.locale, dateFormat)}`
    } : undefined, pickupRT ? {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.arrival"
      }),
      value: `${departure.label} • ${(0, _utils.format)(pickupRTEndDate, intl.locale, dateFormat)}`
    } : undefined, pickupRT ? {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.flightNumber"
      }),
      value: pickupRT.flightNumber
    } : undefined].filter(o => o)
  };
};

const toRendableExcursion = (intl, inscription) => {
  const {
    id,
    price,
    product: {
      paxplan,
      options,
      excursion
    }
  } = inscription;
  const paxCount = paxplan.reduce((acc, pax) => acc + pax.value, 0);
  return {
    id,
    label: intl.formatMessage({
      id: "transfer.excursion"
    }),
    pax: `${paxCount} ${intl.formatMessage({
      id: "transfer.people"
    })}`,
    price: (0, _utils.formatPrice)(intl, price),
    subLabel: [{
      type: "normal",
      text: `${excursion.departureLabel} ➔ ${excursion.title}`
    }],
    details: [{
      type: "details",
      label: intl.formatMessage({
        id: "transfer.date"
      }),
      value: (0, _utils.format)(excursion.departureDate, intl.locale)
    }, {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.place"
      }),
      value: excursion.meetingPoint
    }, {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.steps"
      }),
      value: excursion.poi.map(etape => etape.label).join(", ")
    }, options && options.length > 0 ? {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.included"
      }),
      value: options.map(option => `${option.value} ${option.label}`).join(", ")
    } : undefined].filter(o => o)
  };
};

const toRendableShuttle = (intl, inscription) => {
  const {
    id,
    price,
    product: {
      paxplan,
      origin,
      destination,
      roundTrip,
      departureDate,
      arrivalDate,
      departureDateRT,
      arrivalDateRT,
      number,
      numberRT,
      luggage,
      luggageRT
    }
  } = inscription;
  const paxCount = paxplan.reduce((acc, pax) => acc + pax.value, 0);
  return {
    id,
    label: intl.formatMessage({
      id: "transfer.shuttle"
    }),
    pax: `${paxCount} ${intl.formatMessage({
      id: "transfer.people"
    })}`,
    price: (0, _utils.formatPrice)(intl, price),
    subLabel: [{
      type: "normal",
      text: roundTrip ? intl.formatMessage({
        id: "transfer.roundTrip"
      }) : intl.formatMessage({
        id: "transfer.oneWay"
      })
    }],
    details: [roundTrip ? {
      type: "title",
      label: intl.formatMessage({
        id: "transfer.forwardPath"
      })
    } : undefined, {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.departure"
      }),
      value: `${origin.label} • ${(0, _utils.format)(departureDate, intl.locale, dateFormat)}`
    }, {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.arrival"
      }),
      value: `${destination.label} • ${(0, _utils.format)(arrivalDate, intl.locale, dateFormat)}`
    }, {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.flightNumber"
      }),
      value: number
    }, luggage > 0 ? {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.extraLuggages"
      }),
      value: `${luggage} ${intl.formatMessage({
        id: "transfer.luggage"
      })}`
    } : undefined, roundTrip ? {
      type: "title",
      label: intl.formatMessage({
        id: "transfer.returnTrip"
      })
    } : undefined, roundTrip ? {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.departure"
      }),
      value: `${destination.label} • ${(0, _utils.format)(departureDateRT, intl.locale, dateFormat)}`
    } : undefined, roundTrip ? {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.arrival"
      }),
      value: `${origin.label} • ${(0, _utils.format)(arrivalDateRT, intl.locale, dateFormat)}`
    } : undefined, roundTrip ? {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.flightNumber"
      }),
      value: numberRT
    } : undefined, roundTrip && luggageRT > 0 ? {
      type: "details",
      label: intl.formatMessage({
        id: "transfer.extraLuggages"
      }),
      value: `${luggageRT} ${intl.formatMessage({
        id: "transfer.luggage"
      })}`
    } : undefined].filter(o => o)
  };
};

const renderInscription = {
  EXCURSION: toRendableExcursion,
  PRIVATE_TRANSFER: toRendablePrivateTransfer,
  SHUTTLE: toRendableShuttle
};

const toRendableTransfer = (intl, transfer) => {
  const {
    merchant,
    merchantName,
    inscriptions
  } = transfer;
  return {
    code: merchant,
    name: merchantName,
    inscriptions: inscriptions.map(inscription => renderInscription[inscription.kind](intl, inscription))
  };
};

exports.toRendableTransfer = toRendableTransfer;