"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "sql", {
  enumerable: true,
  get: function () {
    return _sql.default;
  }
});
exports.initConnection = exports.insert = exports.executeSingle = exports.execute = exports.executeQuery = exports.getConnection = void 0;

var _mysql = _interopRequireDefault(require("mysql"));

var _config = _interopRequireDefault(require("config"));

var _util = _interopRequireDefault(require("util"));

var _logger = _interopRequireDefault(require("./logger"));

var _sql = _interopRequireDefault(require("./sql.js"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const databases = _config.default.get("databases");

const createPool = conf => {
  return _mysql.default.createPool({ ...conf,
    typeCast: (field, next) => {
      if (field.type === "JSON") {
        try {
          return JSON.parse(field.string());
        } catch (e) {}
      }

      return next();
    }
  });
};

const pools = Object.entries(databases).reduce((acc, [k, v]) => {
  const pool = createPool(v);
  pool.query = _util.default.promisify(pool.query);
  pool.getConnection = _util.default.promisify(pool.getConnection);
  return { ...acc,
    [k]: pool
  };
}, {});

const getPool = (database = "default") => {
  if (pools[database] === undefined) {
    const errorMessage = `Unknown database "${database}" specified`;

    _logger.default.error(errorMessage);

    throw new Error(errorMessage);
  }

  return pools[database];
};

const getConnection = async database => {
  const pool = getPool(database);
  const connection = await pool.getConnection();
  connection.query = _util.default.promisify(connection.query);
  connection.beginTransaction = _util.default.promisify(connection.beginTransaction);
  connection.commit = _util.default.promisify(connection.commit);
  connection.rollback = _util.default.promisify(connection.rollback);
  return connection;
};

exports.getConnection = getConnection;

const executeQuery = (database, defaultValue, transformation) => {
  const pool = getPool(database);
  return async query => {
    try {
      const result = await pool.query(query);
      return transformation !== undefined ? transformation(result, defaultValue) : result;
    } catch (err) {
      _logger.default.error(err);

      return typeof defaultValue === "function" ? undefined : defaultValue;
    }
  };
};

exports.executeQuery = executeQuery;

const toSingle = (result, defaultValue) => {
  const isFunction = typeof defaultValue === "function";
  return result.length === 0 ? isFunction ? defaultValue() : defaultValue : isFunction ? defaultValue(result[0]) : result[0];
};

const insertId = result => {
  return result !== undefined ? result.insertId : undefined;
};

const execute = database => {
  return executeQuery(database, []);
};

exports.execute = execute;

const executeSingle = (database, defaultValue) => {
  return executeQuery(database, defaultValue, toSingle);
};

exports.executeSingle = executeSingle;

const insert = database => {
  return executeQuery(database, undefined, insertId);
};

exports.insert = insert;

const initConnection = async database => {
  const connection = await getConnection(database);
  await connection.beginTransaction();
  return connection;
};

exports.initConnection = initConnection;