"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.forURL = void 0;

var _sharp = _interopRequireDefault(require("sharp"));

var _nodeFetch = _interopRequireDefault(require("node-fetch"));

var _fs = require("fs");

var _path = _interopRequireDefault(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const CACHE_DURATION = 1000 * 60 * 60; // 1 hour

const PIXEL_DENSITY = 2;
const DEFAULT_OPTIONS = {
  store: "/images",
  format: "png",
  height: 100
};

const getPathForUrl = async (url, options) => {
  const root = "/tmp" + options.store;
  const parsedUrl = new URL(url);
  const filePath = parsedUrl.pathname;

  const parsedPath = _path.default.parse(filePath);

  const localDir = root + parsedPath.dir;
  const fileName = `${parsedPath.name}.${options.format}`;
  await _fs.promises.mkdir(localDir, {
    recursive: true
  });
  return _path.default.join(root, parsedPath.dir, fileName);
};

const cachedUrl = async (url, options) => {
  const path = await getPathForUrl(url, options);

  try {
    const stat = await _fs.promises.stat(path);
    const now = new Date().getTime();
    const expiresAt = stat.mtimeMs + CACHE_DURATION;

    if (now < expiresAt) {
      const info = await (0, _sharp.default)(path).metadata();
      const width = info.width / PIXEL_DENSITY;
      const height = info.height / PIXEL_DENSITY;
      return [path, width, height];
    }

    return undefined;
  } catch (err) {
    return undefined;
  }
};

const computeWidth = (infos, options) => {
  const {
    width: iw,
    height: ih
  } = infos;
  const {
    width: ow,
    height: oh
  } = options;
  return ow !== null && ow !== void 0 ? ow : Math.round(iw / ih * oh);
};

const computeHeight = (infos, options) => {
  const {
    width: iw,
    height: ih
  } = infos;
  const {
    width: ow,
    height: oh
  } = options;
  return oh !== null && oh !== void 0 ? oh : Math.round(ih / iw * ow);
};

const loadImage = async url => {
  const response = await (0, _nodeFetch.default)(url);

  if (response.status !== 200) {
    return undefined;
  }

  return response.buffer();
};

const cacheUrl = async (url, options) => {
  const path = await getPathForUrl(url, options);
  const buffer = await loadImage(url);
  const info = await (0, _sharp.default)(buffer).metadata();
  const density = 72 * 100 / info.height;
  const width = computeWidth(info, options);
  const height = computeHeight(info, options);
  await (0, _sharp.default)(buffer, {
    density
  }).resize(width * PIXEL_DENSITY, height * PIXEL_DENSITY).toFile(path);
  return [path, width, height];
};

const forURL = async (url, options = {}) => {
  const opts = { ...DEFAULT_OPTIONS,
    ...options
  };
  const cached = await cachedUrl(url, opts);
  return cached !== null && cached !== void 0 ? cached : cacheUrl(url, opts);
};

exports.forURL = forURL;